from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List
from discord import EntityType

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.scheduled_event_repository_abc import ScheduledEventRepositoryABC
from bot_data.model.scheduled_event import ScheduledEvent
from bot_data.model.scheduled_event_interval_enum import ScheduledEventIntervalEnum


class ScheduledEventRepositoryService(ScheduledEventRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        servers: ServerRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context

        self._servers = servers

        ScheduledEventRepositoryABC.__init__(self)

    @staticmethod
    def _get_value_from_result(value: any) -> Optional[any]:
        if isinstance(value, str) and "NULL" in value:
            return None

        return value

    def _scheduled_event_from_result(self, sql_result: tuple) -> ScheduledEvent:
        return ScheduledEvent(
            self._get_value_from_result(ScheduledEventIntervalEnum(sql_result[1])),  # interval
            self._get_value_from_result(sql_result[2]),  # name
            self._get_value_from_result(sql_result[3]),  # description
            int(self._get_value_from_result(sql_result[4])),  # channel_id
            self._get_value_from_result(sql_result[5]),  # start_time
            self._get_value_from_result(sql_result[6]),  # end_time
            EntityType(int(self._get_value_from_result(sql_result[7]))),  # entity_type
            self._get_value_from_result(sql_result[8]),  # location
            self._servers.get_server_by_id((sql_result[9])),  # server
            self._get_value_from_result(sql_result[10]),  # created_at
            self._get_value_from_result(sql_result[11]),  # modified_at
            id=self._get_value_from_result(sql_result[0]),
        )

    def get_scheduled_events(self) -> List[ScheduledEvent]:
        scheduled_events = List(ScheduledEvent)
        self._logger.trace(__name__, f"Send SQL command: {ScheduledEvent.get_select_all_string()}")
        results = self._context.select(ScheduledEvent.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get scheduled_event with id {result[0]}")
            scheduled_events.append(self._scheduled_event_from_result(result))

        return scheduled_events

    def get_scheduled_event_by_id(self, id: int) -> ScheduledEvent:
        self._logger.trace(__name__, f"Send SQL command: {ScheduledEvent.get_select_by_id_string(id)}")
        result = self._context.select(ScheduledEvent.get_select_by_id_string(id))[0]

        return self._scheduled_event_from_result(result)

    def get_scheduled_events_by_server_id(self, server_id: int) -> List[ScheduledEvent]:
        scheduled_events = List(ScheduledEvent)
        self._logger.trace(
            __name__,
            f"Send SQL command: {ScheduledEvent.get_select_by_server_id_string(server_id)}",
        )
        results = self._context.select(ScheduledEvent.get_select_by_server_id_string(server_id))

        for result in results:
            self._logger.trace(__name__, f"Get scheduled_event with id {result[0]}")
            scheduled_events.append(self._scheduled_event_from_result(result))

        return scheduled_events

    def add_scheduled_event(self, scheduled_event: ScheduledEvent):
        self._logger.trace(__name__, f"Send SQL command: {scheduled_event.insert_string}")
        self._context.cursor.execute(scheduled_event.insert_string)

    def update_scheduled_event(self, scheduled_event: ScheduledEvent):
        self._logger.trace(__name__, f"Send SQL command: {scheduled_event.udpate_string}")
        self._context.cursor.execute(scheduled_event.udpate_string)

    def delete_scheduled_event(self, scheduled_event: ScheduledEvent):
        self._logger.trace(__name__, f"Send SQL command: {scheduled_event.delete_string}")
        self._context.cursor.execute(scheduled_event.delete_string)
